import * as d3 from 'd3';
// documentation https://github.com/d3/d3/blob/main/API.md

function renderData(dataElements) {

    let width = 500;
    let height = 400;
    let states = new Set(dataElements.map(o => o.State));
    let fullData = dataElements.map(o => ({ year: +o.Year, state: o.State, rate: +o.Abortion_Rate }));
    let data = fullData.filter(el => el.year === 2010);
    data = data.map(el => ({ year: el.year, state: el.state, rate: el.rate }));
    let rts = new Set(data.map(o => o.rate));
    let margin = ({ top: 25, right: 20, bottom: 35, left: 40 });

    // colors for the states
    const color = d3.scaleSequential()
        .domain(d3.extent(rts))
        .range(["lightblue", d3.schemeCategory10[0]]);

    let x = d3.scaleBand()
        .domain(states)
        .range([margin.left, width - margin.right])
        .paddingInner(0.15)
        .paddingOuter(0.15);

    let y = d3.scaleLinear()
        .domain([0, d3.max(rts)]).nice()
        .range([height - margin.bottom, margin.top]);

    // function with g arg assigned to xAxis
    let xAxis = g => g
        .attr("transform", `translate(0,${height - margin.bottom})`)
        .call(d3.axisBottom(x));

    // function with g arg assigned to yAxis
    let yAxis = g => g
        .attr("transform", `translate(${margin.left},0)`)
        .call(d3.axisLeft(y));

    let svg = d3.select("body")
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom);

    // legend
    let legend = d3.legendColor()
        .scale(color)
        .shape("path", d3.symbol().type(d3.symbolSquare).size(150)())
        .title("Rate");

    svg.append("g")
        .attr("class", "legend")
        .attr("transform", `translate(${width - 10}, ${margin.top + 30})`);

    svg.append("g")
        .call(xAxis);

    svg.append("g")
        .call(yAxis);

    svg.select(".legend")
        .style("font-size", "12px")
        .call(legend);

    // div for the tooltip
    var tooltip = d3.select("body").append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    svg.append("g").selectAll("rect")
        .data(data)
        .join("rect")
        .attr("fill", function(d) { return color(d.rate); })
        .attr("width", x.bandwidth())
        .attr("height", d => (height - margin.bottom - y(d.rate)))
        .attr("x", function(d) { return x(d.state); })
        .attr("y", function(d) { return y(d.rate); })
        .on("mouseover", function(d, data) {
            tooltip.transition()
                .duration(200)
                .style("opacity", .85);

            tooltip.style("position", "absolute")
                .style("text-align", "center")
                .style("padding", "5px")
                .style("font-size", "12.5px")
                .style("background", "rgb(231, 231, 229)")
                .style("border", "0px")
                .style("border-radius", "6px")
                .style("pointer-events", "none");

            tooltip.html("In " + data.year + " abortion rate was " + data.rate + ".")
                .style("left", d3.select(this).attr("x") + "px")
                .style("top", d3.select(this).attr("y") + "px");
        })
        .on("mouseout", function(d) {
            tooltip.transition()
                .duration(500)
                .style("opacity", 0);
        });

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width / 4.5)
        .attr("y", "20")
        .style("font-size", "18px")
        .text("Abortion Rates for Group 15-17 Years Old in 2010");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width / 4.5 + 100)
        .attr("y", height - 5)
        .style("font-size", "14px")
        .text("State");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", height / 4)
        .attr("y", "-10")
        .attr("transform", "translate(30, 300) rotate(270)")
        .style("font-size", "14px")
        .text("Abortion Rate");
}

d3.csv(
    "https://gist.githubusercontent.com/olga-kondr/8c0189b181cf74dea61c96b669e60260/raw/c3ec5a222078115fcef9c98bcc9869a2e1d0b329/ab_rate.csv"
).then(data => renderData(data));